@@ -32,7 +32,6 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <string.h>
-#include <dbus/dbus-glib.h>
 #include <gdk/gdkx.h>
 
 #include "nautilus-file-attributes.h"
@@ -94,6 +93,7 @@ typedef struct {
 #define RESPONSE_MARK_TRUSTED 1003
 
 #define SILENT_WINDOW_OPEN_LIMIT 5
+#define SILENT_OPEN_LIMIT 5
 
 /* This number controls a maximum character count for a URL that is
  * displayed as part of a dialog. It's fairly arbitrary -- big enough
@@ -1140,6 +1140,8 @@ typedef struct {
 	char *activation_directory;
 	gboolean user_confirmation;
 	char *uri;
+	GDBusProxy *proxy;
+	GtkWidget *dialog;
 } ActivateParametersInstall;
 
 static void
@@ -1155,6 +1157,7 @@ activate_parameters_install_free (ActivateParametersInstall *parameters_install)
 	nautilus_file_list_free (parameters_install->files);
 	g_free (parameters_install->activation_directory);
 	g_free (parameters_install->uri);
+        g_object_unref (parameters_install->proxy);
 	g_free (parameters_install);
 }
 
@@ -1300,71 +1303,55 @@ show_unhandled_type_error (ActivateParametersInstall *parameters)
 }
 
 static void
-search_for_application_dbus_call_notify_cb (DBusGProxy *proxy, DBusGProxyCall *call, ActivateParametersInstall *parameters_install)
+search_for_application_dbus_call_notify_cb (GDBusProxy   *proxy,
+					    GAsyncResult *result,
+					    gpointer      user_data)
 {
-	gboolean ret;
+	ActivateParametersInstall *parameters_install = user_data;
+	GVariant *variant;
 	GError *error = NULL;
-	char *message;
-	const char *remote = NULL;
 
-	ret = dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_INVALID);
-	if (!ret) {
-		if (error->domain == DBUS_GERROR && error->code == DBUS_GERROR_REMOTE_EXCEPTION) {
-			remote = dbus_g_error_get_name (error);
-		}
-		/* we already show an error in the installer if not found, just catch generic failure */
-		if (remote == NULL || strcmp (remote, "org.freedesktop.PackageKit.Modify.Failed") == 0) {
-			message = g_strdup_printf ("%s\n%s",
-						   _("There was an internal error trying to search for applications:"),
-						   error->message);
-			eel_show_error_dialog (_("Unable to search for application"), message,
-					       parameters_install->parent_window);
-			g_free (message);
-		}
+	variant = g_dbus_proxy_call_finish (proxy, result, &error);
+	if (variant == NULL) {
+		if (!g_dbus_error_is_remote_error (error) ||
+		    g_strcmp0 (g_dbus_error_get_remote_error (error), "org.freedesktop.PackageKit.Modify.Failed") == 0) {
+			    char *message;
+
+			    message = g_strdup_printf ("%s\n%s",
+						       _("There was an internal error trying to search for applications:"),
+						       error->message);
+			    eel_show_error_dialog (_("Unable to search for application"), message,
+			                           parameters_install->parent_window);
+			    g_free (message);
+		    }
+
 		g_error_free (error);
+		activate_parameters_install_free (parameters_install);
 		return;
 	}
-	g_object_unref (proxy);
+
+	g_variant_unref (variant);
 
 	/* activate the file again */
 	nautilus_mime_activate_files (parameters_install->parent_window,
-				      parameters_install->slot_info,
-				      parameters_install->files,
-				      parameters_install->activation_directory,
-				      parameters_install->mode,
-				      parameters_install->flags,
-				      parameters_install->user_confirmation);
+	                              parameters_install->slot_info,
+	                              parameters_install->files,
+	                              parameters_install->activation_directory,
+	                              parameters_install->mode,
+	                              parameters_install->flags,
+	                              parameters_install->user_confirmation);
 
-	/* parameters_install freed by destroy notify */
+	activate_parameters_install_free (parameters_install);
 }
 
 static void
 search_for_application_mime_type (ActivateParametersInstall *parameters_install, const gchar *mime_type)
 {
-	DBusGConnection *connection;
-	DBusGProxy *proxy = NULL;
-	guint xid = 0;
-	GError *error = NULL;
-	DBusGProxyCall *call = NULL;
-	GtkWidget *dialog;
 	GdkWindow *window;
+	guint xid = 0;
 	const char *mime_types[2];
 
-	/* get bus */
-	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-	if (connection == NULL) {
-		g_error_free (error);
-		goto out;
-	}
-
-	/* get proxy - native clients for for KDE and GNOME */
-	proxy = dbus_g_proxy_new_for_name (connection,
-					   "org.freedesktop.PackageKit",
-					   "/org/freedesktop/PackageKit",
-					   "org.freedesktop.PackageKit.Modify");
-	if (proxy == NULL) {
-		goto out;
-	}
+	g_assert (parameters_install->proxy != NULL);	
 
 	/* get XID from parent window */
 	window = gtk_widget_get_window (GTK_WIDGET (parameters_install->parent_window));
@@ -1372,48 +1359,34 @@ search_for_application_mime_type (ActivateParametersInstall *parameters_install,
 		xid = GDK_WINDOW_XID (window);
 	}
 
-	/* don't timeout, as dbus-glib sets the timeout ~25 seconds */
-	dbus_g_proxy_set_default_timeout (proxy, INT_MAX);
-
-	/* invoke the method */
 	mime_types[0] = mime_type;
 	mime_types[1] = NULL;
-	call = dbus_g_proxy_begin_call (proxy, "InstallMimeTypes",
-					(DBusGProxyCallNotify) search_for_application_dbus_call_notify_cb,
-					parameters_install,
-					(GDestroyNotify) activate_parameters_install_free,
-				        G_TYPE_UINT, xid,
-				        G_TYPE_STRV, mime_types,
-				        G_TYPE_STRING, "hide-confirm-search",
-				        G_TYPE_INVALID);
-	if (call == NULL) {
-		dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						 _("Could not use system package installer"));
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy), NULL);
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		gtk_widget_show (dialog);
-		goto out;
-	}
+
+	g_dbus_proxy_call (parameters_install->proxy,
+			   "InstallMimeTypes",
+			   g_variant_new ("(u^ass)",
+					  xid,
+					  mime_types,
+					  "hide-confirm-search"),
+			   G_DBUS_CALL_FLAGS_NONE,
+			   G_MAXINT /* no timeout */,
+			   NULL /* cancellable */,
+			   (GAsyncReadyCallback) search_for_application_dbus_call_notify_cb,
+			   parameters_install);
 
 	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-			    "InstallMimeType method invoked for %s", mime_type);
-out:
-	if (call == NULL) {
-		/* dbus method was not called, so we're not going to get the async dbus callback */
-		activate_parameters_install_free (parameters_install);
-	}
+	                    "InstallMimeType method invoked for %s", mime_type);
 }
 
 static void
-application_unhandled_file_install (GtkDialog *dialog, gint response_id, ActivateParametersInstall *parameters_install)
+application_unhandled_file_install (GtkDialog *dialog,
+                                    gint response_id,
+                                    ActivateParametersInstall *parameters_install)
 {
 	char *mime_type;
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
+	parameters_install->dialog = NULL;
 
 	if (response_id == GTK_RESPONSE_YES) {
 		mime_type = nautilus_file_get_mime_type (parameters_install->file);
@@ -1425,49 +1398,64 @@ application_unhandled_file_install (GtkDialog *dialog, gint response_id, Activat
 	}
 }
 
-static void
-packagekit_present_dbus_call_notify_cb (DBusGProxy *proxy, DBusGProxyCall *call, ActivateParametersInstall *parameters_install)
+static gboolean
+delete_cb (GtkDialog *dialog)
 {
-	gboolean ret;
-	GError *error = NULL;
+	gtk_dialog_response (dialog, GTK_RESPONSE_DELETE_EVENT);
+	return TRUE;
+}
+
+static void
+pk_proxy_appeared_cb (GObject *source,
+		      GAsyncResult *res,
+		      gpointer user_data)
+{
+        ActivateParametersInstall *parameters_install = user_data;
 	char *mime_type;
 	char *error_message;
-	gboolean present;
 	GtkWidget *dialog;
+        GDBusProxy *proxy;
+	GError *error = NULL;
 
+	proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 
-	ret = dbus_g_proxy_end_call (proxy, call, &error, G_TYPE_BOOLEAN, &present, G_TYPE_INVALID);
-	if (!ret) {
+	if (error != NULL) {
+		g_warning ("Couldn't call Modify on the PackageKit interface: %s",
+			   error->message);
 		g_error_free (error);
+
+		/* show an unhelpful dialog */
 		show_unhandled_type_error (parameters_install);
+		/* The callback wasn't started, so we have to free the parameters */
 		activate_parameters_install_free (parameters_install);
-		present = FALSE;
-		goto out;
-	}
-	if (!present) {
-		show_unhandled_type_error (parameters_install);
-		activate_parameters_install_free (parameters_install);
-		goto out;
+
+		return;
 	}
 
 	mime_type = nautilus_file_get_mime_type (parameters_install->file);
-	error_message = get_application_no_mime_type_handler_message (parameters_install->file, parameters_install->uri);
+	error_message = get_application_no_mime_type_handler_message (parameters_install->file,
+	                                                              parameters_install->uri);
 	/* use a custom dialog to prompt the user to install new software */
-	dialog = gtk_message_dialog_new (NULL, 0,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_YES_NO,
-					 "%s", error_message);
+	dialog = gtk_message_dialog_new (parameters_install->parent_window, 0,
+	                                 GTK_MESSAGE_ERROR,
+	                                 GTK_BUTTONS_YES_NO,
+	                                 "%s", error_message);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("There is no application installed for %s files.\nDo you want to search for an application to open this file?"),
-						  g_content_type_get_description (mime_type));
+	                                          _("There is no application installed for %s files.\n"
+	                                            "Do you want to search for an application to open this file?"),
+	                                          g_content_type_get_description (mime_type));
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 
-	g_signal_connect (dialog, "response", G_CALLBACK (application_unhandled_file_install), parameters_install);
+	parameters_install->dialog = dialog;
+	parameters_install->proxy = proxy;
+
+	g_signal_connect (dialog, "response",
+	                  G_CALLBACK (application_unhandled_file_install),
+	                  parameters_install);
+	g_signal_connect (dialog, "delete-event",
+	                  G_CALLBACK (delete_cb), NULL);
 	gtk_widget_show_all (dialog);
 	g_free (mime_type);
-
-out:
-	g_object_unref (proxy);
 }
 
 static void
@@ -1477,10 +1465,6 @@ application_unhandled_uri (ActivateParameters *parameters, char *uri)
 	char *mime_type;
 	NautilusFile *file;
 	ActivateParametersInstall *parameters_install;
-	DBusGConnection *connection;
-	DBusGProxy *proxy;
-	DBusGProxyCall *call;
-	GError *error = NULL;
 
 	file = nautilus_file_get_by_uri (uri);
 
@@ -1495,7 +1479,7 @@ application_unhandled_uri (ActivateParameters *parameters, char *uri)
 		g_object_add_weak_pointer (G_OBJECT (parameters_install->parent_window), (gpointer *)&parameters_install->parent_window);
 	}
 	parameters_install->activation_directory = g_strdup (parameters->activation_directory);
-	parameters_install->file = nautilus_file_ref (file);
+	parameters_install->file = file;
 	parameters_install->files = get_file_list_for_launch_locations (parameters->locations);
 	parameters_install->mode = parameters->mode;
 	parameters_install->flags = parameters->flags;
@@ -1512,49 +1496,31 @@ application_unhandled_uri (ActivateParameters *parameters, char *uri)
 	/* There is no use trying to look for handlers of application/octet-stream */
 	if (g_content_type_is_unknown (mime_type)) {
 		show_install_mime = FALSE;
+		goto out;
 	}
 
 	if (!show_install_mime) {
 		goto out;
 	}
 
-	/* Check whether PackageKit can be spawned */
-	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-	if (connection == NULL) {
-		g_error_free (error);
-		show_install_mime = FALSE;
-		goto out;
-	}
+	g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+				  G_DBUS_PROXY_FLAGS_NONE,
+				  NULL,
+				  "org.freedesktop.PackageKit",
+				  "/org/freedesktop/PackageKit",
+				  "org.freedesktop.PackageKit.Modify",
+				  NULL,
+				  pk_proxy_appeared_cb,
+				  parameters_install);
 
-	proxy = dbus_g_proxy_new_for_name (connection,
-					   DBUS_SERVICE_DBUS,
-					   DBUS_PATH_DBUS,
-					   DBUS_INTERFACE_DBUS);
-
-	if (proxy == NULL) {
-		show_install_mime = FALSE;
-		goto out;
-	}
-	call = dbus_g_proxy_begin_call (proxy, "NameHasOwner",
-					(DBusGProxyCallNotify) packagekit_present_dbus_call_notify_cb,
-					parameters_install,
-					NULL, /* Don't want to free user_data as we need to pass it on */
-				        G_TYPE_STRING, "org.freedesktop.PackageKit",
-				        G_TYPE_INVALID);
-	if (call == NULL) {
-		show_install_mime = FALSE;
-		goto out;
-	}
+	return;
 
 out:
-	if (!show_install_mime) {
-		/* show an unhelpful dialog */
-		show_unhandled_type_error (parameters_install);
-		/* The callback wasn't started, so we have to free the parameters */
-		activate_parameters_install_free (parameters_install);
-	}
+        /* show an unhelpful dialog */
+        show_unhandled_type_error (parameters_install);
+        /* The callback wasn't started, so we have to free the parameters */
+        activate_parameters_install_free (parameters_install);
 
-	nautilus_file_unref (file);
 	g_free (mime_type);
 }
 
@@ -1696,6 +1662,10 @@ activate_files (ActivateParameters *parameters)
 	ActivationAction action;
 	GdkScreen *screen;
 	LaunchLocation *location;
+	gint num_apps;
+	gint num_unhandled;
+	gint num_files;
+	gboolean open_files;
 	
 	screen = gtk_widget_get_screen (GTK_WIDGET (parameters->parent_window));
 
@@ -1862,20 +1832,53 @@ activate_files (ActivateParameters *parameters)
 			(open_in_app_uris, &unhandled_open_in_app_uris);
 	}
 
-	for (l = open_in_app_parameters; l != NULL; l = l->next) {
-		one_parameters = l->data;
+	num_apps = g_list_length (open_in_app_parameters);
+	num_unhandled = g_list_length (unhandled_open_in_app_uris);
+	num_files = g_list_length (open_in_app_uris);
+	open_files = TRUE;
 
-		nautilus_launch_application_by_uri (one_parameters->application,
-						    one_parameters->uris,
-						    parameters->parent_window);
-		application_launch_parameters_free (one_parameters);
+	if (!parameters->user_confirmation || num_files + num_unhandled > SILENT_OPEN_LIMIT) {
+		GtkDialog *dialog;
+		char *prompt;
+		char *detail;
+		int response;
+
+		pause_activation_timed_cancel (parameters);
+
+		prompt = _("Are you sure you want to open all files?");
+		detail = g_strdup_printf (ngettext ("This will open %d separate application.",
+						    "This will open %d separate applications.", num_apps), num_apps);
+		dialog = eel_show_yes_no_dialog (prompt, detail,
+						 GTK_STOCK_OK, GTK_STOCK_CANCEL,
+						 parameters->parent_window);
+		g_free (detail);
+
+		response = gtk_dialog_run (dialog);
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+
+		unpause_activation_timed_cancel (parameters);
+
+		if (response != GTK_RESPONSE_YES) {
+			open_files = FALSE;
+		}
 	}
 
-	for (l = unhandled_open_in_app_uris; l != NULL; l = l->next) {
-		uri = l->data;
+	if (open_files) {
+		for (l = open_in_app_parameters; l != NULL; l = l->next) {
+			one_parameters = l->data;
 
-		/* this does not block */
-		application_unhandled_uri (parameters, uri);
+			nautilus_launch_application_by_uri (one_parameters->application,
+							    one_parameters->uris,
+							    parameters->parent_window);
+			application_launch_parameters_free (one_parameters);
+		}
+
+		for (l = unhandled_open_in_app_uris; l != NULL; l = l->next) {
+			uri = l->data;
+
+			/* this does not block */
+			application_unhandled_uri (parameters, uri);
+		}
 	}
 
 	window_info = NULL;
@@ -1958,6 +1961,7 @@ activation_mount_not_mounted (ActivateParameters *parameters)
 	if (parameters->not_mounted != NULL) {
 		file = parameters->not_mounted->data;
 		mount_op = gtk_mount_operation_new (parameters->parent_window);
+		g_mount_operation_set_password_save (mount_op, G_PASSWORD_SAVE_FOR_SESSION);
 		g_signal_connect (mount_op, "notify::is-showing",
 				  G_CALLBACK (activate_mount_op_active), parameters);
 		location = nautilus_file_get_location (file);
@@ -2218,6 +2222,7 @@ activation_mount_mountables (ActivateParameters *parameters)
 	if (parameters->mountables != NULL) {
 		file = parameters->mountables->data;
 		mount_op = gtk_mount_operation_new (parameters->parent_window);
+		g_mount_operation_set_password_save (mount_op, G_PASSWORD_SAVE_FOR_SESSION);
 		g_signal_connect (mount_op, "notify::is-showing",
 				  G_CALLBACK (activate_mount_op_active), parameters);
 		nautilus_file_mount (file,
