@@ -22,9 +22,8 @@
    Authors: Anders Carlsson <andersca@gnu.org>
 */
 
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtktreeview.h>
-#include <gdk/gdkdnd.h>
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include <libnautilus-private/nautilus-file.h>
 #include <libnautilus-private/nautilus-directory.h>
 #include <libnautilus-extension/nautilus-column.h>
@@ -32,11 +31,17 @@
 #ifndef FM_LIST_MODEL_H
 #define FM_LIST_MODEL_H
 
-#define FM_TYPE_LIST_MODEL		(fm_list_model_get_type ())
-#define FM_LIST_MODEL(obj)		(GTK_CHECK_CAST ((obj), FM_TYPE_LIST_MODEL, FMListModel))
-#define FM_LIST_MODEL_CLASS(klass)	(GTK_CHECK_CLASS_CAST ((klass), FM_TYPE_LIST_MODEL, FMListModelClass))
-#define FM_IS_LIST_MODEL(obj)		(GTK_CHECK_TYPE ((obj), FM_TYPE_LIST_MODEL))
-#define FM_IS_LIST_MODEL_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), FM_TYPE_LIST_MODEL))
+#define FM_TYPE_LIST_MODEL fm_list_model_get_type()
+#define FM_LIST_MODEL(obj) \
+  (G_TYPE_CHECK_INSTANCE_CAST ((obj), FM_TYPE_LIST_MODEL, FMListModel))
+#define FM_LIST_MODEL_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_CAST ((klass), FM_TYPE_LIST_MODEL, FMListModelClass))
+#define FM_IS_LIST_MODEL(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FM_TYPE_LIST_MODEL))
+#define FM_IS_LIST_MODEL_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_TYPE ((klass), FM_TYPE_LIST_MODEL))
+#define FM_LIST_MODEL_GET_CLASS(obj) \
+  (G_TYPE_INSTANCE_GET_CLASS ((obj), FM_TYPE_LIST_MODEL, FMListModelClass))
 
 enum {
 	FM_LIST_MODEL_FILE_COLUMN,
@@ -119,8 +124,8 @@ void              fm_list_model_set_drag_view (FMListModel *model,
 					       int begin_x, 
 					       int begin_y);
 
-void              fm_list_model_get_drag_types (const GtkTargetEntry **entries,
-						int *num_entries);
+GtkTargetList *   fm_list_model_get_drag_target_list (void);
+
 int               fm_list_model_compare_func (FMListModel *model,
 					      NautilusFile *file1,
 					      NautilusFile *file2);
@@ -133,5 +138,8 @@ int               fm_list_model_get_column_number (FMListModel *model,
 						   
 void              fm_list_model_subdirectory_done_loading (FMListModel       *model,
 							   NautilusDirectory *directory);
+
+void              fm_list_model_set_highlight_for_files (FMListModel *model,
+                                                         GList *files);
 						   
 #endif /* FM_LIST_MODEL_H */
