@@ -56,11 +56,11 @@
 #include "nautilus-window-private.h"
 #include "nautilus-window-manage-views.h"
 #include <unistd.h>
+#include <errno.h>
 #include <libxml/xmlsave.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
-#include <dbus/dbus-glib.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
@@ -77,14 +77,10 @@
 #include <libnautilus-extension/nautilus-menu-provider.h>
 #include <libnautilus-private/nautilus-autorun.h>
 
-enum
-{
-  COMMAND_0, /* unused: 0 is an invalid command */
-
-  COMMAND_START_DESKTOP,
-  COMMAND_STOP_DESKTOP,
-  COMMAND_OPEN_BROWSER,
-};
+#define COMMAND_OPEN "open"
+#define COMMAND_START_DESKTOP "start_desktop"
+#define COMMAND_STOP_DESKTOP "stop_desktop"
+#define COMMAND_CLOSE "close"
 
 /* Keep window from shrinking down ridiculously small; numbers are somewhat arbitrary */
 #define APPLICATION_WINDOW_MIN_WIDTH	300
@@ -124,80 +120,9 @@ static void     drive_listen_for_eject_button      (GDrive *drive,
 						    NautilusApplication *application);
 static void     nautilus_application_load_session     (NautilusApplication *application); 
 static char *   nautilus_application_get_session_data (void);
-static void     ck_session_active_changed_cb (DBusGProxy *proxy,
-		                              gboolean is_active,
-                		              void *user_data);
-
 
 G_DEFINE_TYPE (NautilusApplication, nautilus_application, G_TYPE_OBJECT);
 
-static gboolean
-_unique_message_data_set_geometry_and_uris (UniqueMessageData  *message_data,
-					    const char *geometry,
-					    char **uris)
-{
-  GString *list;
-  gint i;
-  gchar *result;
-  gsize length;
-
-  list = g_string_new (NULL);
-  if (geometry != NULL) {
-	  g_string_append (list, geometry);
-  }
-  g_string_append (list, "\r\n");
-  
-  for (i = 0; uris != NULL && uris[i]; i++) {
-	  g_string_append (list, uris[i]);
-	  g_string_append (list, "\r\n");
-  }
-
-  result = g_convert (list->str, list->len,
-                      "ASCII", "UTF-8",
-                      NULL, &length, NULL);
-  g_string_free (list, TRUE);
-  
-  if (result) {
-	  unique_message_data_set (message_data, (guchar *) result, length);
-	  g_free (result);
-	  return TRUE;
-  }
-  
-  return FALSE;
-}
-
-static gchar **
-_unique_message_data_get_geometry_and_uris (UniqueMessageData *message_data,
-					    char **geometry)
-{
-  gchar **result = NULL;
-
-  *geometry = NULL;
-  
-  gchar *text, *newline, *uris;
-  text = unique_message_data_get_text (message_data);
-  if (text) {
-	  newline = strchr (text, '\n');
-	  if (newline) {
-		  *geometry = g_strndup (text, newline-text);
-		  uris = newline+1;
-	  } else {
-		  uris = text;
-	  }
-	  
-	  result = g_uri_list_extract_uris (uris);
-	  g_free (text);
-  }
-
-  /* if the string is empty, make it NULL */
-  if (*geometry && strlen (*geometry) == 0) {
-	  g_free (*geometry);
-	  *geometry = NULL;
-  }
-
-  return result;
-}
-
 GList *
 nautilus_application_get_window_list (void)
 {
@@ -287,13 +212,6 @@ nautilus_application_init (NautilusApplication *application)
 	/* Create an undo manager */
 	application->undo_manager = nautilus_undo_manager_new ();
 
-	application->unique_app = unique_app_new_with_commands ("org.gnome.Nautilus", NULL,
-								"start_desktop", COMMAND_START_DESKTOP,
-								"stop_desktop", COMMAND_STOP_DESKTOP,
-								"open_browser", COMMAND_OPEN_BROWSER,
-								NULL);
-
-	
         application->smclient = egg_sm_client_get ();
         g_signal_connect (application->smclient, "save_state",
                           G_CALLBACK (smclient_save_state_cb),
@@ -357,11 +275,9 @@ nautilus_application_finalize (GObject *object)
 		application->automount_idle_id = 0;
 	}
 
-	if (application->ck_session_proxy != NULL) {
-		dbus_g_proxy_disconnect_signal (application->ck_session_proxy, "ActiveChanged",
-						G_CALLBACK (ck_session_active_changed_cb), NULL);
-		g_object_unref (application->ck_session_proxy);
-		application->ck_session_proxy = NULL;
+	if (application->proxy != NULL) {
+		g_object_unref (application->proxy);
+		application->proxy = NULL;
 	}
 
         G_OBJECT_CLASS (nautilus_application_parent_class)->finalize (object);
@@ -529,106 +445,153 @@ mark_desktop_files_trusted (void)
 	g_free (do_once_file);
 }
 
-#define CK_NAME "org.freedesktop.ConsoleKit"
-#define CK_PATH "/org/freedesktop/ConsoleKit"
+#define CK_NAME       "org.freedesktop.ConsoleKit"
+#define CK_PATH       "/org/freedesktop/ConsoleKit"
+#define CK_INTERFACE  "org.freedesktop.ConsoleKit"
 
 static void
-ck_session_active_changed_cb (DBusGProxy *proxy,
-			      gboolean is_active,
-			      void *user_data)
+ck_session_proxy_signal_cb (GDBusProxy *proxy,
+			    const char *sender_name,
+			    const char *signal_name,
+			    GVariant   *parameters,
+			    gpointer    user_data)
 {
 	NautilusApplication *application = user_data;
 
-	application->session_is_active = is_active;
+	if (g_strcmp0 (signal_name, "ActiveChanged") == 0) {
+		g_variant_get (parameters, "(b)", &application->session_is_active);
+	}
 }
 
 static void
-ck_call_is_active_cb (DBusGProxy *proxy,
-		      DBusGProxyCall *call_id,
-		      void *user_data)
+ck_call_is_active_cb (GDBusProxy   *proxy,
+		      GAsyncResult *result,
+		      gpointer      user_data)
 {
-	gboolean res, is_active;
-	NautilusApplication *application;
+	NautilusApplication *application = user_data;
+	GVariant *variant;
+	GError *error = NULL;
 
-	application = user_data;
+	variant = g_dbus_proxy_call_finish (proxy, result, &error);
 
-	res = dbus_g_proxy_end_call (proxy, call_id, NULL,
-				     G_TYPE_BOOLEAN, &is_active,
-				     G_TYPE_INVALID);
-	if (!res) {
-		g_object_unref (proxy);
+	if (variant == NULL) {
+		g_warning ("Error when calling IsActive(): %s\n", error->message);
+		application->session_is_active = TRUE;
+
+		g_error_free (error);
+		return;
+	}
+
+	g_variant_get (variant, "(b)", &application->session_is_active);
+
+	g_variant_unref (variant);
+}
+
+static void
+session_proxy_appeared (GObject       *source,
+                        GAsyncResult *res,
+                        gpointer      user_data)
+{
+	NautilusApplication *application = user_data;
+        GDBusProxy *proxy;
+	GError *error = NULL;
+
+        proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
+
+	if (error != NULL) {
+		g_warning ("Failed to get the current CK session: %s", error->message);
+		g_error_free (error);
 
 		application->session_is_active = TRUE;
 		return;
 	}
 
-	application->session_is_active = is_active;
+	g_signal_connect (proxy, "g-signal",
+			  G_CALLBACK (ck_session_proxy_signal_cb),
+			  application);
 
-	dbus_g_proxy_add_signal (proxy, "ActiveChanged", G_TYPE_BOOLEAN, G_TYPE_INVALID);
-	dbus_g_proxy_connect_signal (proxy, "ActiveChanged",
-				     G_CALLBACK (ck_session_active_changed_cb), application,
-				     NULL);
+	g_dbus_proxy_call (proxy,
+			   "IsActive",
+			   g_variant_new ("()"),
+			   G_DBUS_CALL_FLAGS_NONE,
+			   -1,
+			   NULL,
+			   (GAsyncReadyCallback) ck_call_is_active_cb,
+			   application);
+
+        application->proxy = proxy;
 }
 
 static void
-ck_get_current_session_cb (DBusGProxy *proxy,
-			   DBusGProxyCall *call_id,
-			   void *user_data)
+ck_get_current_session_cb (GDBusConnection *connection,
+			   GAsyncResult    *result,
+			   gpointer         user_data)
 {
-	gboolean res;
-	char *session_id;
-	NautilusApplication *application;
+	NautilusApplication *application = user_data;
+	GVariant *variant;
+	const char *session_path = NULL;
+	GError *error = NULL;
 
-	application = user_data;
+	variant = g_dbus_connection_call_finish (connection, result, &error);
 
-	res = dbus_g_proxy_end_call (proxy, call_id, NULL,
-				     DBUS_TYPE_G_OBJECT_PATH, &session_id, G_TYPE_INVALID);
-	if (!res) {
-		g_object_unref (proxy);
+	if (variant == NULL) {
+		g_warning ("Failed to get the current CK session: %s", error->message);
+		g_error_free (error);
 
 		application->session_is_active = TRUE;
 		return;
 	}
 
-	application->ck_session_proxy = dbus_g_proxy_new_from_proxy (proxy, CK_NAME ".Session",
-								     session_id);
-	dbus_g_proxy_begin_call (application->ck_session_proxy, "IsActive", ck_call_is_active_cb,
-				 application, NULL, G_TYPE_INVALID);
+	g_variant_get (variant, "(&o)", &session_path);
 
-	g_free (session_id);
-	g_object_unref (proxy);
+	g_dbus_proxy_new_for_bus (G_BUS_TYPE_SYSTEM,
+				  G_DBUS_PROXY_FLAGS_NONE,
+				  NULL,
+				  CK_NAME,
+				  session_path,
+				  CK_INTERFACE ".Session",
+				  NULL,
+				  session_proxy_appeared,
+				  application);
+
+	g_variant_unref (variant);
 }
 
-
 static void
 do_initialize_consolekit (NautilusApplication *application)
 {
-	DBusGConnection *conn;
-	DBusGProxy *proxy;
-	GError *error = NULL;
+	GDBusConnection *connection;
 
-	conn = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
-	if (error) {
-		g_error_free (error);
+	connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, NULL);
 
+	if (connection == NULL) {
 		application->session_is_active = TRUE;
-
 		return;
 	}
 
-	proxy = dbus_g_proxy_new_for_name (conn, CK_NAME, CK_PATH "/Manager",
- 					   CK_NAME ".Manager");
-	dbus_g_proxy_begin_call (proxy, "GetCurrentSession",
-				 ck_get_current_session_cb, application,
-				 NULL, G_TYPE_INVALID);
+	g_dbus_connection_call (connection,
+				CK_NAME,
+				CK_PATH "/Manager",
+				CK_INTERFACE ".Manager",
+				"GetCurrentSession",
+				g_variant_new ("()"),
+				G_VARIANT_TYPE ("(o)"),
+				G_DBUS_CALL_FLAGS_NONE,
+				-1,
+				NULL,
+				(GAsyncReadyCallback) ck_get_current_session_cb,
+				application);
+
+	g_object_unref (connection);
 }
 
 static void
 do_upgrades_once (NautilusApplication *application,
 		  gboolean no_desktop)
 {
-	char *metafile_dir, *updated;
-	int fd;
+	char *metafile_dir, *updated, *nautilus_dir, *xdg_dir;
+	const gchar *message;
+	int fd, res;
 
 	if (!no_desktop) {
 		mark_desktop_files_trusted ();
@@ -648,6 +611,35 @@ do_upgrades_once (NautilusApplication *application,
 		g_free (updated);
 	}
 	g_free (metafile_dir);
+
+	nautilus_dir = g_build_filename (g_get_home_dir (),
+					 ".nautilus", NULL);
+	xdg_dir = nautilus_get_user_directory ();
+	if (g_file_test (nautilus_dir, G_FILE_TEST_IS_DIR)) {
+		/* test if we already attempted to migrate first */
+		updated = g_build_filename (nautilus_dir, "DEPRECATED-DIRECTORY", NULL);
+		message = _("Nautilus 2.32 deprecated this directory and tried migrating "
+			    "this configuration to ~/.config/nautilus");
+		if (!g_file_test (updated, G_FILE_TEST_EXISTS)) {
+			/* rename() works fine if the destination directory is
+			 * empty.
+			 */
+			res = g_rename (nautilus_dir, xdg_dir);
+
+			if (res == -1) {
+				fd = g_creat (updated, 0600);
+				if (fd != -1) {
+					res = write (fd, message, strlen (message));
+					close (fd);
+				}
+			}
+		}
+
+		g_free (updated);
+	}
+
+	g_free (nautilus_dir);
+	g_free (xdg_dir);
 }
 
 static void
@@ -770,53 +762,65 @@ open_windows (NautilusApplication *application,
 	}
 }
 
-static UniqueResponse
-message_received_cb (UniqueApp         *unique_app,
-                     UniqueCommand      command,
-                     UniqueMessageData *message,
-                     guint              time_,
-                     gpointer           user_data)
+static void
+message_received_cb (GApplication *unique_app,
+		     gchar *name,
+		     GVariant *platform_data,
+		     gpointer user_data)
 {
 	NautilusApplication *application;
-	UniqueResponse res;
-	char **uris;
-	char *geometry;
-	GdkScreen *screen;
-	
+
 	application =  user_data;
-	res = UNIQUE_RESPONSE_OK;
-	
-	switch (command) {
-	case UNIQUE_CLOSE:
-		res = UNIQUE_RESPONSE_OK;
+
+	if (g_strcmp0 (name, COMMAND_CLOSE) == 0) {
 		nautilus_main_event_loop_quit (TRUE);
-		
-		break;
-	case UNIQUE_OPEN:
-	case COMMAND_OPEN_BROWSER:
-		uris = _unique_message_data_get_geometry_and_uris (message, &geometry);
-		screen = unique_message_data_get_screen (message);
+		return;
+	}
+
+	if (g_strcmp0 (name, COMMAND_OPEN) == 0) {
+		GVariantIter iter;
+		gboolean use_browser;
+		gchar *key;
+		GVariant *value;
+		gchar *startup_id = NULL, *geometry = NULL;
+		char **uris = NULL;
+
+		g_variant_iter_init (&iter, platform_data);
+
+		while (g_variant_iter_next (&iter, "{&sv}", &key, &value)) {
+			if (g_strcmp0 (key, "urls") == 0) {
+				g_variant_get (value, "^as", &uris);
+			} else if (g_strcmp0 (key, "geometry") == 0) {
+				g_variant_get (value, "&s", &geometry);
+			} else if (g_strcmp0 (key, "use_browser") == 0) {
+				g_variant_get (value, "b", &use_browser);
+			} else if (g_strcmp0 (key, "startup_id") == 0) {
+				g_variant_get (value, "&s", &startup_id);
+			}
+
+			g_variant_unref (value);
+		}
+
 		open_windows (application,
-			      unique_message_data_get_startup_id (message),
+			      startup_id,
 			      uris,
-			      screen,
+			      gdk_screen_get_default (),
 			      geometry,
-			      command == COMMAND_OPEN_BROWSER);
+			      use_browser);
 		g_strfreev (uris);
-		g_free (geometry);
-		break;
-	case COMMAND_START_DESKTOP:
+
+		return;
+	}
+
+	if (g_strcmp0 (name, COMMAND_START_DESKTOP) == 0) {
 		nautilus_application_open_desktop (application);
-		break;
-	case COMMAND_STOP_DESKTOP:
+		return;
+	}
+
+	if (g_strcmp0 (name, COMMAND_STOP_DESKTOP) == 0) {
 		nautilus_application_close_desktop ();
-		break;
-	default:
-		res = UNIQUE_RESPONSE_PASSTHROUGH;
-		break;
+		return;
 	}
-	
-	return res;
 }
 
 gboolean 
@@ -848,6 +852,34 @@ queue_accel_map_save_callback (GtkAccelMap *object, gchar *accel_path,
 	}
 }
 
+static GVariant *
+get_startup_id (void)
+{
+	gchar *id = NULL;
+	GVariant *retval;
+	GTimeVal timeval = { 0, };
+
+	id = g_strdup (gdk_x11_display_get_startup_notification_id (gdk_display_get_default ()));
+
+	if (id == NULL) {
+		id = g_strdup (g_getenv ("DESKTOP_STARTUP_ID"));
+	}
+
+	if (id == NULL) {
+		guint32 timestamp;
+
+		g_get_current_time (&timeval);
+
+		timestamp =  timeval.tv_sec;
+		id = g_strdup_printf ("_TIME%lu", (unsigned long) timestamp);
+        }
+
+	retval = g_variant_new ("s", id);
+	g_free (id);
+
+	return retval;
+}
+
 void
 nautilus_application_startup (NautilusApplication *application,
 			      gboolean kill_shell,
@@ -857,8 +889,9 @@ nautilus_application_startup (NautilusApplication *application,
 			      const char *geometry,
 			      char **urls)
 {
-	UniqueMessageData *message;
-	
+	gboolean is_remote;
+	GError *error = NULL;
+
 	/* Check the user's ~/.nautilus directories and post warnings
 	 * if there are problems.
 	 */
@@ -866,11 +899,31 @@ nautilus_application_startup (NautilusApplication *application,
 		return;
 	}
 
+	application->unique_app = g_initable_new (G_TYPE_APPLICATION,
+						  NULL,
+						  &error,
+						  "application-id", "org.gnome.Nautilus",
+						  "default-quit", FALSE,
+						  NULL);
+	g_assert (error == NULL);
+	is_remote = g_application_is_remote (application->unique_app);
+
+	if (!is_remote) {
+		/* set application properties */
+		g_application_add_action (application->unique_app,
+					  "open", "Open a nautilus window");
+		g_application_add_action (application->unique_app,
+					  "close", "Close the application");
+		g_application_add_action (application->unique_app,
+					  COMMAND_START_DESKTOP, "Starts the desktop");
+		g_application_add_action (application->unique_app,
+					  COMMAND_STOP_DESKTOP, "Stops the desktop");
+	}
+
 	if (kill_shell) {
-		if (unique_app_is_running (application->unique_app)) {
-			unique_app_send_message (application->unique_app,
-						 UNIQUE_CLOSE, NULL);
-			
+		if (is_remote) {
+			g_application_invoke_action (application->unique_app,
+						     COMMAND_CLOSE, NULL);
 		}
 	} else {
 		char *accel_map_filename;
@@ -881,19 +934,21 @@ nautilus_application_startup (NautilusApplication *application,
 		}
 			
 		if (!no_desktop) {
-			if (unique_app_is_running (application->unique_app)) {
-				unique_app_send_message (application->unique_app,
-							 COMMAND_START_DESKTOP, NULL);
+			if (is_remote) {
+				g_application_invoke_action (application->unique_app,
+							     COMMAND_START_DESKTOP, NULL);
 			} else {
 				nautilus_application_open_desktop (application);
 			}
 		}
 
-		if (!unique_app_is_running (application->unique_app)) {
+		if (!is_remote) {
 			finish_startup (application, no_desktop);
-			g_signal_connect (application->unique_app, "message-received", G_CALLBACK (message_received_cb), application);			
+			g_signal_connect (application->unique_app,
+					  "action-with-data",
+					  G_CALLBACK (message_received_cb), application);			
 		}
-		
+
 		/* Monitor the preference to show or hide the desktop */
 		eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_SHOW_DESKTOP,
 							  desktop_changed_callback,
@@ -909,17 +964,31 @@ nautilus_application_startup (NautilusApplication *application,
 
 	  	/* Create the other windows. */
 		if (urls != NULL || !no_default_window) {
-			if (unique_app_is_running (application->unique_app)) {
-				message = unique_message_data_new ();
-				_unique_message_data_set_geometry_and_uris (message, geometry, urls);
-				if (browser_window) {
-					unique_app_send_message (application->unique_app,
-								 COMMAND_OPEN_BROWSER, message);
-				} else {
-					unique_app_send_message (application->unique_app,
-								 UNIQUE_OPEN, message);
+			if (is_remote) {
+				GVariant *variant;
+				GVariantBuilder builder;
+
+				g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
+
+				if (geometry != NULL) {
+					g_variant_builder_add (&builder, "{sv}",
+							       "geometry", g_variant_new ("s", geometry));
 				}
-				unique_message_data_free (message);				
+
+				if (urls != NULL) {
+					g_variant_builder_add (&builder, "{sv}",
+							       "urls", g_variant_new ("^as", urls));
+				}
+
+				g_variant_builder_add (&builder, "{sv}",
+						       "use_browser", g_variant_new ("b", browser_window));
+				g_variant_builder_add (&builder, "{sv}",
+						       "startup_id", get_startup_id ());
+				variant = g_variant_builder_end (&builder);
+
+				g_application_invoke_action (application->unique_app,
+							     COMMAND_OPEN, variant);
+				g_variant_unref (variant);
 			} else {
 				open_windows (application, NULL,
 					      urls,
@@ -977,7 +1046,7 @@ get_desktop_manager_selection (GdkDisplay *display, int screen)
 	if (gtk_selection_owner_set_for_display (display,
 						 selection_widget,
 						 selection_atom,
-						 gdk_x11_get_server_time (selection_widget->window))) {
+						 gdk_x11_get_server_time (gtk_widget_get_window (selection_widget)))) {
 		
 		g_signal_connect (selection_widget, "selection_get",
 				  G_CALLBACK (selection_get_cb), NULL);
@@ -1236,8 +1305,6 @@ create_window (NautilusApplication *application,
 						  "app", application,
 						  "screen", screen,
 						  NULL));
-	/* Must be called after construction finished */
-	nautilus_window_constructed (window);
 
 	if (startup_id) {
 		gtk_window_set_startup_id (GTK_WINDOW (window), startup_id);
@@ -1413,8 +1480,8 @@ nautilus_application_create_navigation_window (NautilusApplication *application,
 		eel_gtk_window_set_initial_geometry_from_string 
 			(GTK_WINDOW (window), 
 			 geometry_string,
-			 NAUTILUS_WINDOW_MIN_WIDTH, 
-			 NAUTILUS_WINDOW_MIN_HEIGHT,
+			 NAUTILUS_NAVIGATION_WINDOW_MIN_WIDTH,
+			 NAUTILUS_NAVIGATION_WINDOW_MIN_HEIGHT,
 			 another_navigation_window_already_showing (window));
 	}
 	g_free (geometry_string);
@@ -1793,22 +1860,26 @@ nautilus_application_get_session_data (void)
 		xmlNewProp (win_node, "type", NAUTILUS_IS_NAVIGATION_WINDOW (window) ? "navigation" : "spatial");
 
 		if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) { /* spatial windows store their state as file metadata */
+			GdkWindow *gdk_window;
+
 			tmp = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
 			xmlNewProp (win_node, "geometry", tmp);
 			g_free (tmp);
 
-			if (GTK_WIDGET (window)->window &&
-			    gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_MAXIMIZED) {
+			gdk_window = gtk_widget_get_window (GTK_WIDGET (window));
+
+			if (gdk_window &&
+			    gdk_window_get_state (gdk_window) & GDK_WINDOW_STATE_MAXIMIZED) {
 				xmlNewProp (win_node, "maximized", "TRUE");
 			}
 
-			if (GTK_WIDGET (window)->window &&
-			    gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_STICKY) {
+			if (gdk_window &&
+			    gdk_window_get_state (gdk_window) & GDK_WINDOW_STATE_STICKY) {
 				xmlNewProp (win_node, "sticky", "TRUE");
 			}
 
-			if (GTK_WIDGET (window)->window &&
-			    gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_ABOVE) {
+			if (gdk_window &&
+			    gdk_window_get_state (gdk_window) & GDK_WINDOW_STATE_ABOVE) {
 				xmlNewProp (win_node, "keep-above", "TRUE");
 			}
 		}
@@ -1971,8 +2042,8 @@ nautilus_application_load_session (NautilusApplication *application)
 						eel_gtk_window_set_initial_geometry_from_string 
 							(GTK_WINDOW (window), 
 							 geometry,
-							 NAUTILUS_WINDOW_MIN_WIDTH, 
-							 NAUTILUS_WINDOW_MIN_HEIGHT,
+							 NAUTILUS_NAVIGATION_WINDOW_MIN_WIDTH,
+							 NAUTILUS_NAVIGATION_WINDOW_MIN_HEIGHT,
 							 FALSE);
 					}
 					xmlFree (geometry);
